@@ -44,6 +44,8 @@ import org.h2.util.New;
  * ALTER TABLE ALTER COLUMN SET DEFAULT,
  * ALTER TABLE ALTER COLUMN SET NOT NULL,
  * ALTER TABLE ALTER COLUMN SET NULL,
+ * ALTER TABLE ALTER COLUMN SET VISIBLE,
+ * ALTER TABLE ALTER COLUMN SET INVISIBLE,
  * ALTER TABLE DROP COLUMN
  */
 public class AlterTableAlterColumn extends SchemaCommand {
@@ -60,6 +62,7 @@ public class AlterTableAlterColumn extends SchemaCommand {
     private boolean ifNotExists;
     private ArrayList<Column> columnsToAdd;
     private ArrayList<Column> columnsToRemove;
+    private boolean newVisibility;
 
     public AlterTableAlterColumn(Session session, Schema schema) {
         super(session, schema);
@@ -156,9 +159,13 @@ public class AlterTableAlterColumn extends SchemaCommand {
                 } else if (!oldColumn.isNullable() && newColumn.isNullable()) {
                     checkNullable(table);
                 }
+                if (oldColumn.getVisible() ^ newColumn.getVisible()) {
+                    oldColumn.setVisible(newColumn.getVisible());
+                }
                 convertAutoIncrementColumn(table, newColumn);
                 copyData(table);
             }
+            table.setModified();
             break;
         }
         case CommandInterface.ALTER_TABLE_ADD_COLUMN: {
@@ -192,13 +199,19 @@ public class AlterTableAlterColumn extends SchemaCommand {
             db.updateMeta(session, table);
             break;
         }
+        case CommandInterface.ALTER_TABLE_ALTER_COLUMN_VISIBILITY: {
+            oldColumn.setVisible(newVisibility);
+            table.setModified();
+            db.updateMeta(session, table);
+            break;
+        }
         default:
             DbException.throwInternalError("type=" + type);
         }
         return 0;
     }
 
-    private void checkDefaultReferencesTable(Table table, Expression defaultExpression) {
+    private static void checkDefaultReferencesTable(Table table, Expression defaultExpression) {
         if (defaultExpression == null) {
             return;
         }
@@ -550,4 +563,8 @@ public class AlterTableAlterColumn extends SchemaCommand {
     public void setColumnsToRemove(ArrayList<Column> columnsToRemove) {
         this.columnsToRemove = columnsToRemove;
     }
+
+    public void setVisible(boolean visible) {
+        this.newVisibility = visible;
+    }
 }
