@@ -12,11 +12,13 @@ import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.Map;
 import java.util.Random;
+import java.util.concurrent.TimeUnit;
 import org.h2.api.ErrorCode;
 import org.h2.command.Command;
 import org.h2.command.CommandInterface;
 import org.h2.command.Parser;
 import org.h2.command.Prepared;
+import org.h2.command.ddl.Analyze;
 import org.h2.command.dml.Query;
 import org.h2.command.dml.SetTypes;
 import org.h2.constraint.Constraint;
@@ -29,7 +31,7 @@ import org.h2.message.TraceSystem;
 import org.h2.mvstore.db.MVTable;
 import org.h2.mvstore.db.TransactionStore.Change;
 import org.h2.mvstore.db.TransactionStore.Transaction;
-import org.h2.result.LocalResult;
+import org.h2.result.ResultInterface;
 import org.h2.result.Row;
 import org.h2.result.SortOrder;
 import org.h2.schema.Schema;
@@ -39,6 +41,7 @@ import org.h2.store.LobStorageFrontend;
 import org.h2.table.SubQueryInfo;
 import org.h2.table.Table;
 import org.h2.table.TableFilter;
+import org.h2.table.TableType;
 import org.h2.util.New;
 import org.h2.util.SmallLRUCache;
 import org.h2.value.Value;
@@ -78,13 +81,14 @@ public class Session extends SessionWithState {
     private int lockTimeout;
     private Value lastIdentity = ValueLong.get(0);
     private Value lastScopeIdentity = ValueLong.get(0);
+    private Value lastTriggerIdentity;
     private int firstUncommittedLog = Session.LOG_WRITTEN;
     private int firstUncommittedPos = Session.LOG_WRITTEN;
     private HashMap<String, Savepoint> savepoints;
     private HashMap<String, Table> localTempTables;
     private HashMap<String, Index> localTempTableIndexes;
     private HashMap<String, Constraint> localTempTableConstraints;
-    private int throttle;
+    private long throttleNs;
     private long lastThrottle;
     private Command currentCommand;
     private boolean allowLiterals;
@@ -98,13 +102,13 @@ public class Session extends SessionWithState {
     private boolean redoLogBinary = true;
     private boolean autoCommitAtTransactionEnd;
     private String currentTransactionName;
-    private volatile long cancelAt;
+    private volatile long cancelAtNs;
     private boolean closed;
     private final long sessionStart = System.currentTimeMillis();
     private long transactionStart;
     private long currentCommandStart;
     private HashMap<String, Value> variables;
-    private HashSet<LocalResult> temporaryResults;
+    private HashSet<ResultInterface> temporaryResults;
     private int queryTimeout;
     private boolean commitOrRollbackDisabled;
     private Table waitForLock;
@@ -121,6 +125,12 @@ public class Session extends SessionWithState {
     private HashMap<Object, ViewIndex> subQueryIndexCache;
     private boolean joinBatchEnabled;
     private boolean forceJoinOrder;
+    private boolean lazyQueryExecution;
+    /**
+     * Tables marked for ANALYZE after the current transaction is committed.
+     * Prevents us calling ANALYZE repeatedly in large transactions.
+     */
+    private HashSet<Table> tablesToAnalyze;
 
     /**
      * Temporary LOBs from result sets. Those are kept for some time. The
@@ -154,6 +164,14 @@ public class Session extends SessionWithState {
         this.currentSchemaName = Constants.SCHEMA_MAIN;
     }
 
+    public void setLazyQueryExecution(boolean lazyQueryExecution) {
+        this.lazyQueryExecution = lazyQueryExecution;
+    }
+
+    public boolean isLazyQueryExecution() {
+        return lazyQueryExecution;
+    }
+
     public void setForceJoinOrder(boolean forceJoinOrder) {
         this.forceJoinOrder = forceJoinOrder;
     }
@@ -646,9 +664,23 @@ public class Session extends SessionWithState {
             }
         }
         endTransaction();
+
+        int rows = getDatabase().getSettings().analyzeSample / 10;
+        if (tablesToAnalyze != null) {
+            for (Table table : tablesToAnalyze) {
+                Analyze.analyzeTable(this, table, rows, false);
+            }
+        }
+        tablesToAnalyze = null;
     }
 
     private void removeTemporaryLobs(boolean onTimeout) {
+        if (SysProperties.CHECK2) {
+            if (this == getDatabase().getLobSession()
+                    && !Thread.holdsLock(this) && !Thread.holdsLock(getDatabase())) {
+                throw DbException.throwInternalError();
+            }
+        }
         if (temporaryLobs != null) {
             for (Value v : temporaryLobs) {
                 if (!v.isLinkedToTable()) {
@@ -658,8 +690,8 @@ public class Session extends SessionWithState {
             temporaryLobs.clear();
         }
         if (temporaryResultLobs != null && temporaryResultLobs.size() > 0) {
-            long keepYoungerThan = System.currentTimeMillis() -
-                    database.getSettings().lobTimeout;
+            long keepYoungerThan = System.nanoTime() -
+                    TimeUnit.MILLISECONDS.toNanos(database.getSettings().lobTimeout);
             while (temporaryResultLobs.size() > 0) {
                 TimeoutValue tv = temporaryResultLobs.getFirst();
                 if (onTimeout && tv.created >= keepYoungerThan) {
@@ -801,7 +833,7 @@ public class Session extends SessionWithState {
 
     @Override
     public void cancel() {
-        cancelAt = System.currentTimeMillis();
+        cancelAtNs = System.nanoTime();
     }
 
     @Override
@@ -809,6 +841,10 @@ public class Session extends SessionWithState {
         if (!closed) {
             try {
                 database.checkPowerOff();
+
+                // release any open table locks
+                rollback();
+
                 removeTemporaryLobs(false);
                 cleanTempTables(true);
                 undoLog.clear();
@@ -828,7 +864,7 @@ public class Session extends SessionWithState {
     public void addLock(Table table) {
         if (SysProperties.CHECK) {
             if (locks.contains(table)) {
-                DbException.throwInternalError();
+                DbException.throwInternalError(table.toString());
             }
         }
         locks.add(table);
@@ -853,11 +889,11 @@ public class Session extends SessionWithState {
                 int lockMode = database.getLockMode();
                 if (lockMode != Constants.LOCK_MODE_OFF &&
                         !database.isMultiVersion()) {
-                    String tableType = log.getTable().getTableType();
+                    TableType tableType = log.getTable().getTableType();
                     if (locks.indexOf(log.getTable()) < 0
-                            && !Table.TABLE_LINK.equals(tableType)
-                            && !Table.EXTERNAL_TABLE_ENGINE.equals(tableType)) {
-                        DbException.throwInternalError();
+                            && TableType.TABLE_LINK != tableType
+                            && TableType.EXTERNAL_TABLE_ENGINE != tableType) {
+                        DbException.throwInternalError("" + tableType);
                     }
                 }
             }
@@ -927,11 +963,13 @@ public class Session extends SessionWithState {
     private void cleanTempTables(boolean closeSession) {
         if (localTempTables != null && localTempTables.size() > 0) {
             synchronized (database) {
-                for (Table table : New.arrayList(localTempTables.values())) {
+                Iterator<Table> it = localTempTables.values().iterator();
+                while (it.hasNext()) {
+                    Table table = it.next();
                     if (closeSession || table.getOnCommitDrop()) {
                         modificationId++;
                         table.setModified();
-                        localTempTables.remove(table.getName());
+                        it.remove();
                         table.removeChildrenAndResources(this);
                         if (closeSession) {
                             // need to commit, otherwise recovery might
@@ -942,6 +980,11 @@ public class Session extends SessionWithState {
                         table.truncate(this);
                     }
                 }
+                // sometimes Table#removeChildrenAndResources
+                // will take the meta lock
+                if (closeSession) {
+                    database.unlockMeta(this);
+                }
             }
         }
     }
@@ -983,6 +1026,14 @@ public class Session extends SessionWithState {
         return lastScopeIdentity;
     }
 
+    public void setLastTriggerIdentity(Value last) {
+        this.lastTriggerIdentity = last;
+    }
+
+    public Value getLastTriggerIdentity() {
+        return lastTriggerIdentity;
+    }
+
     /**
      * Called when a log entry for this session is added. The session keeps
      * track of the first entry in the transaction log that is not yet
@@ -1116,7 +1167,7 @@ public class Session extends SessionWithState {
     }
 
     public void setThrottle(int throttle) {
-        this.throttle = throttle;
+        this.throttleNs = TimeUnit.MILLISECONDS.toNanos(throttle);
     }
 
     /**
@@ -1126,16 +1177,16 @@ public class Session extends SessionWithState {
         if (currentCommandStart == 0) {
             currentCommandStart = System.currentTimeMillis();
         }
-        if (throttle == 0) {
+        if (throttleNs == 0) {
             return;
         }
-        long time = System.currentTimeMillis();
-        if (lastThrottle + Constants.THROTTLE_DELAY > time) {
+        long time = System.nanoTime();
+        if (lastThrottle + TimeUnit.MILLISECONDS.toNanos(Constants.THROTTLE_DELAY) > time) {
             return;
         }
-        lastThrottle = time + throttle;
+        lastThrottle = time + throttleNs;
         try {
-            Thread.sleep(throttle);
+            Thread.sleep(TimeUnit.NANOSECONDS.toMillis(throttleNs));
         } catch (Exception e) {
             // ignore InterruptedException
         }
@@ -1150,9 +1201,9 @@ public class Session extends SessionWithState {
     public void setCurrentCommand(Command command) {
         this.currentCommand = command;
         if (queryTimeout > 0 && command != null) {
-            long now = System.currentTimeMillis();
-            currentCommandStart = now;
-            cancelAt = now + queryTimeout;
+            currentCommandStart = System.currentTimeMillis();
+            long now = System.nanoTime();
+            cancelAtNs = now + TimeUnit.MILLISECONDS.toNanos(queryTimeout);
         }
     }
 
@@ -1164,12 +1215,12 @@ public class Session extends SessionWithState {
      */
     public void checkCanceled() {
         throttle();
-        if (cancelAt == 0) {
+        if (cancelAtNs == 0) {
             return;
         }
-        long time = System.currentTimeMillis();
-        if (time >= cancelAt) {
-            cancelAt = 0;
+        long time = System.nanoTime();
+        if (time >= cancelAtNs) {
+            cancelAtNs = 0;
             throw DbException.get(ErrorCode.STATEMENT_WAS_CANCELED);
         }
     }
@@ -1180,7 +1231,7 @@ public class Session extends SessionWithState {
      * @return the time or 0 if not set
      */
     public long getCancel() {
-        return cancelAt;
+        return cancelAtNs;
     }
 
     public Command getCurrentCommand() {
@@ -1204,10 +1255,17 @@ public class Session extends SessionWithState {
         this.currentSchemaName = schema.getName();
     }
 
+    @Override
     public String getCurrentSchemaName() {
         return currentSchemaName;
     }
 
+    @Override
+    public void setCurrentSchemaName(String schemaName) {
+        Schema schema = database.getSchema(schemaName);
+        setCurrentSchema(schema);
+    }
+
     /**
      * Create an internal connection. This connection is used when initializing
      * triggers, and when calling user defined functions.
@@ -1237,7 +1295,7 @@ public class Session extends SessionWithState {
      */
     public void removeAtCommit(Value v) {
         if (SysProperties.CHECK && !v.isLinkedToTable()) {
-            DbException.throwInternalError();
+            DbException.throwInternalError(v.toString());
         }
         if (removeLobMap == null) {
             removeLobMap = New.hashMap();
@@ -1433,7 +1491,7 @@ public class Session extends SessionWithState {
      *
      * @param result the temporary result set
      */
-    public void addTemporaryResult(LocalResult result) {
+    public void addTemporaryResult(ResultInterface result) {
         if (!result.needToClose()) {
             return;
         }
@@ -1448,7 +1506,7 @@ public class Session extends SessionWithState {
 
     private void closeTemporaryResults() {
         if (temporaryResults != null) {
-            for (LocalResult result : temporaryResults) {
+            for (ResultInterface result : temporaryResults) {
                 result.close();
             }
             temporaryResults = null;
@@ -1464,7 +1522,7 @@ public class Session extends SessionWithState {
         this.queryTimeout = queryTimeout;
         // must reset the cancel at here,
         // otherwise it is still used
-        this.cancelAt = 0;
+        this.cancelAtNs = 0;
     }
 
     public int getQueryTimeout() {
@@ -1632,6 +1690,23 @@ public class Session extends SessionWithState {
         }
     }
 
+    @Override
+    public boolean isRemote() {
+        return false;
+    }
+
+    /**
+     * Mark that the given table needs to be analyzed on commit.
+     *
+     * @param table the table
+     */
+    public void markTableForAnalyze(Table table) {
+        if (tablesToAnalyze == null) {
+            tablesToAnalyze = New.hashSet();
+        }
+        tablesToAnalyze.add(table);
+    }
+
     /**
      * Represents a savepoint (a position in a transaction to where one can roll
      * back to).
@@ -1657,7 +1732,7 @@ public class Session extends SessionWithState {
         /**
          * The time when this object was created.
          */
-        final long created = System.currentTimeMillis();
+        final long created = System.nanoTime();
 
         /**
          * The value.
