@@ -194,6 +194,15 @@ public class Comparison extends Condition {
                             return ValueExpression.getNull();
                         }
                     }
+                    int colType = left.getType();
+                    int constType = r.getType();
+                    int resType = Value.getHigherOrder(colType, constType);
+                    // If not, the column values will need to be promoted
+                    // to constant type, but vise versa, then let's do this here
+                    // once.
+                    if (constType != resType) {
+                        right = ValueExpression.get(r.convertTo(resType));
+                    }
                 } else if (right instanceof Parameter) {
                     ((Parameter) right).setColumn(
                             ((ExpressionColumn) left).getColumn());
@@ -206,7 +215,7 @@ public class Comparison extends Condition {
             }
         } else {
             if (SysProperties.CHECK && (left == null || right == null)) {
-                DbException.throwInternalError();
+                DbException.throwInternalError(left + " " + right);
             }
             if (left == ValueExpression.getNull() ||
                     right == ValueExpression.getNull()) {
