@@ -300,7 +300,7 @@ public class PageDataIndex extends PageIndex {
 
     @Override
     public Cursor findFirstOrLast(Session session, boolean first) {
-        throw DbException.throwInternalError();
+        throw DbException.throwInternalError(toString());
     }
 
     long getLastKey() {
@@ -464,6 +464,11 @@ public class PageDataIndex extends PageIndex {
         return -1;
     }
 
+    @Override
+    public boolean isFirstColumn(Column column) {
+        return false;
+    }
+
     @Override
     public void close(Session session) {
         if (trace.isDebugEnabled()) {
