@@ -23,6 +23,7 @@ import java.io.PrintWriter;
 import java.sql.Connection;
 import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.concurrent.TimeUnit;
 import java.util.logging.Logger;
 import javax.sql.ConnectionEvent;
 import javax.sql.ConnectionEventListener;
@@ -190,7 +191,7 @@ public class JdbcConnectionPool implements DataSource, ConnectionEventListener,
      */
     @Override
     public Connection getConnection() throws SQLException {
-        long max = System.currentTimeMillis() + timeout * 1000;
+        long max = System.nanoTime() + TimeUnit.SECONDS.toNanos(timeout);
         do {
             synchronized (this) {
                 if (activeConnections < maxConnections) {
@@ -202,7 +203,7 @@ public class JdbcConnectionPool implements DataSource, ConnectionEventListener,
                     // ignore
                 }
             }
-        } while (System.currentTimeMillis() <= max);
+        } while (System.nanoTime() <= max);
         throw new SQLException("Login timeout", "08001", 8001);
     }
 
