@@ -174,6 +174,7 @@ public class CacheLongKeyLIRS<V> {
      * @param value the value
      * @return the size
      */
+    @SuppressWarnings("unused")
     protected int sizeOf(V value) {
         return 1;
     }
@@ -791,9 +792,12 @@ public class CacheLongKeyLIRS<V> {
             }
             V old;
             Entry<V> e = find(key, hash);
+            boolean existed;
             if (e == null) {
+                existed = false;
                 old = null;
             } else {
+                existed = true;
                 old = e.value;
                 remove(key, hash);
             }
@@ -822,6 +826,10 @@ public class CacheLongKeyLIRS<V> {
             mapSize++;
             // added entries are always added to the stack
             addToStack(e);
+            if (existed) {
+                // if it was there before (even non-resident), it becomes hot
+                access(key, hash);
+            }
             return old;
         }
 
