@@ -7,6 +7,7 @@ package org.h2.mvstore.db;
 
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
+import java.util.BitSet;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
@@ -45,7 +46,7 @@ public class TransactionStore {
      * is not possible). Log entries are written before the data is changed
      * (write-ahead).
      * <p>
-     * Key: [ opId ], value: [ mapId, key, oldValue ].
+     * Key: opId, value: [ mapId, key, oldValue ].
      */
     final MVMap<Long, Object[]> undoLog;
 
@@ -57,9 +58,9 @@ public class TransactionStore {
 
     private final DataType dataType;
 
-    private boolean init;
+    private final BitSet openTransactions = new BitSet();
 
-    private int lastTransactionId;
+    private boolean init;
 
     private int maxTransactionId = 0xffff;
 
@@ -119,8 +120,10 @@ public class TransactionStore {
         }
         synchronized (undoLog) {
             if (undoLog.size() > 0) {
-                Long key = undoLog.firstKey();
-                lastTransactionId = getTransactionId(key);
+                for (Long key : undoLog.keySet()) {
+                    int transactionId = getTransactionId(key);
+                    openTransactions.set(transactionId);
+                }
             }
         }
     }
@@ -220,16 +223,23 @@ public class TransactionStore {
      * @return the transaction
      */
     public synchronized Transaction begin() {
+
+        int transactionId;
+        int status;
         if (!init) {
             throw DataUtils.newIllegalStateException(
                     DataUtils.ERROR_TRANSACTION_ILLEGAL_STATE,
                     "Not initialized");
         }
-        int transactionId = ++lastTransactionId;
-        if (lastTransactionId >= maxTransactionId) {
-            lastTransactionId = 0;
+        transactionId = openTransactions.nextClearBit(1);
+        if (transactionId > maxTransactionId) {
+            throw DataUtils.newIllegalStateException(
+                    DataUtils.ERROR_TOO_MANY_OPEN_TRANSACTIONS,
+                    "There are {0} open transactions",
+                    transactionId - 1);
         }
-        int status = Transaction.STATUS_OPEN;
+        openTransactions.set(transactionId);
+        status = Transaction.STATUS_OPEN;
         return new Transaction(this, transactionId, status, null, 0);
     }
 
@@ -263,7 +273,7 @@ public class TransactionStore {
             if (logId == 0) {
                 if (undoLog.containsKey(undoKey)) {
                     throw DataUtils.newIllegalStateException(
-                            DataUtils.ERROR_TRANSACTION_STILL_OPEN,
+                            DataUtils.ERROR_TOO_MANY_OPEN_TRANSACTIONS,
                             "An old transaction with the same id " +
                             "is still open: {0}",
                             t.getId());
@@ -280,9 +290,15 @@ public class TransactionStore {
      * @param logId the log id
      */
     public void logUndo(Transaction t, long logId) {
-        long[] undoKey = { t.getId(), logId };
+        Long undoKey = getOperationId(t.getId(), logId);
         synchronized (undoLog) {
-            undoLog.remove(undoKey);
+            Object[] old = undoLog.remove(undoKey);
+            if (old == null) {
+                throw DataUtils.newIllegalStateException(
+                        DataUtils.ERROR_TRANSACTION_ILLEGAL_STATE,
+                        "Transaction {0} was concurrently rolled back",
+                        t.getId());
+            }
         }
     }
 
@@ -334,8 +350,12 @@ public class TransactionStore {
                     if (value == null) {
                         // nothing to do
                     } else if (value.value == null) {
-                        // remove the value
-                        map.remove(key);
+                        int tx = getTransactionId(value.operationId);
+                        if (tx == t.transactionId) {
+                            // remove the value
+                            // only if it's transaction id is same as current transaction id
+                            map.remove(key);
+                        }
                     } else {
                         VersionedValue v2 = new VersionedValue();
                         v2.value = value.value;
@@ -435,6 +455,7 @@ public class TransactionStore {
             preparedTransactions.remove(t.getId());
         }
         t.setStatus(Transaction.STATUS_CLOSED);
+        openTransactions.clear(t.transactionId);
         if (store.getAutoCommitDelay() == 0) {
             store.commit();
             return;
@@ -916,9 +937,11 @@ public class TransactionStore {
                 long size = 0;
                 Cursor<K, VersionedValue> cursor = map.cursor(null);
                 while (cursor.hasNext()) {
-                    K key = cursor.next();
-                    VersionedValue data = cursor.getValue();
-                    data = getValue(key, readLogId, data);
+                    VersionedValue data;
+                    synchronized (transaction.store.undoLog) {
+                        K key = cursor.next();
+                        data = getValue(key, readLogId, cursor.getValue());
+                    }
                     if (data != null && data.value != null) {
                         size++;
                     }
@@ -1180,8 +1203,14 @@ public class TransactionStore {
         }
 
         private VersionedValue getValue(K key, long maxLog) {
-            VersionedValue data = map.get(key);
-            return getValue(key, maxLog, data);
+            synchronized (getUndoLog()) {
+                VersionedValue data = map.get(key);
+                return getValue(key, maxLog, data);
+            }
+        }
+
+        Object getUndoLog() {
+            return transaction.store.undoLog;
         }
 
         /**
@@ -1193,6 +1222,13 @@ public class TransactionStore {
          * @return the value
          */
         VersionedValue getValue(K key, long maxLog, VersionedValue data) {
+            if (MVStore.ASSERT) {
+                if (!Thread.holdsLock(getUndoLog())) {
+                    throw DataUtils.newIllegalStateException(
+                            DataUtils.ERROR_INTERNAL,
+                            "not synchronized on undoLog");
+                }
+            }
             while (true) {
                 if (data == null) {
                     // doesn't exist or deleted by a committed transaction
@@ -1212,41 +1248,21 @@ public class TransactionStore {
                 }
                 // get the value before the uncommitted transaction
                 Object[] d;
-                synchronized (transaction.store.undoLog) {
-                    d = transaction.store.undoLog.get(id);
-                }
+                d = transaction.store.undoLog.get(id);
                 if (d == null) {
+                    if (transaction.store.store.isReadOnly()) {
+                        // uncommitted transaction for a read-only store
+                        return null;
+                    }
                     // this entry should be committed or rolled back
                     // in the meantime (the transaction might still be open)
+                    // or it might be changed again in a different
+                    // transaction (possibly one with the same id)
                     data = map.get(key);
-                    if (data != null && data.operationId == id) {
-                        // the transaction was not committed correctly
-                        throw DataUtils.newIllegalStateException(
-                                DataUtils.ERROR_TRANSACTION_CORRUPT,
-                                "The transaction log might be corrupt for key {0}",
-                                key);
-                    }
                 } else {
                     data = (VersionedValue) d[2];
                 }
-                // verify this is either committed,
-                // or the same transaction and earlier
-                if (data != null) {
-                    long id2 = data.operationId;
-                    if (id2 != 0) {
-                        int tx2 = getTransactionId(id2);
-                        if (tx2 != tx) {
-                            // a different transaction - ok
-                        } else if (getLogId(id2) > getLogId(id)) {
-                            // newer than before
-                            break;
-                        }
-                    }
-                }
             }
-            throw DataUtils.newIllegalStateException(
-                    DataUtils.ERROR_TRANSACTION_CORRUPT,
-                    "The transaction log might be corrupt for key {0}", key);
         }
 
         /**
@@ -1294,23 +1310,6 @@ public class TransactionStore {
             }
         }
 
-        /**
-         * Get the most recent smallest key that is larger or equal to this key.
-         *
-         * @param key the key (may not be null)
-         * @return the result
-         */
-        public K getLatestCeilingKey(K key) {
-            Iterator<K> cursor = map.keyIterator(key);
-            while (cursor.hasNext()) {
-                key = cursor.next();
-                if (get(key, Long.MAX_VALUE) != null) {
-                    return key;
-                }
-            }
-            return null;
-        }
-
         /**
          * Get the smallest key that is larger than the given key, or null if no
          * such key exists.
@@ -1462,37 +1461,39 @@ public class TransactionStore {
 
                 private void fetchNext() {
                     while (cursor.hasNext()) {
-                        K k;
-                        try {
-                            k = cursor.next();
-                        } catch (IllegalStateException e) {
-                            // TODO this is a bit ugly
-                            if (DataUtils.getErrorCode(e.getMessage()) ==
-                                    DataUtils.ERROR_CHUNK_NOT_FOUND) {
-                                cursor = map.cursor(currentKey);
-                                // we (should) get the current key again,
-                                // we need to ignore that one
-                                if (!cursor.hasNext()) {
-                                    break;
-                                }
-                                cursor.next();
-                                if (!cursor.hasNext()) {
-                                    break;
-                                }
+                        synchronized (getUndoLog()) {
+                            K k;
+                            try {
                                 k = cursor.next();
-                            } else {
-                                throw e;
+                            } catch (IllegalStateException e) {
+                                // TODO this is a bit ugly
+                                if (DataUtils.getErrorCode(e.getMessage()) ==
+                                        DataUtils.ERROR_CHUNK_NOT_FOUND) {
+                                    cursor = map.cursor(currentKey);
+                                    // we (should) get the current key again,
+                                    // we need to ignore that one
+                                    if (!cursor.hasNext()) {
+                                        break;
+                                    }
+                                    cursor.next();
+                                    if (!cursor.hasNext()) {
+                                        break;
+                                    }
+                                    k = cursor.next();
+                                } else {
+                                    throw e;
+                                }
+                            }
+                            final K key = k;
+                            VersionedValue data = cursor.getValue();
+                            data = getValue(key, readLogId, data);
+                            if (data != null && data.value != null) {
+                                @SuppressWarnings("unchecked")
+                                final V value = (V) data.value;
+                                current = new DataUtils.MapEntry<K, V>(key, value);
+                                currentKey = key;
+                                return;
                             }
-                        }
-                        final K key = k;
-                        VersionedValue data = cursor.getValue();
-                        data = getValue(key, readLogId, data);
-                        if (data != null && data.value != null) {
-                            @SuppressWarnings("unchecked")
-                            final V value = (V) data.value;
-                            current = new DataUtils.MapEntry<K, V>(key, value);
-                            currentKey = key;
-                            return;
                         }
                     }
                     current = null;
