@@ -6,7 +6,6 @@
 package org.h2.schema;
 
 import java.math.BigInteger;
-
 import org.h2.api.ErrorCode;
 import org.h2.engine.DbObject;
 import org.h2.engine.Session;
@@ -33,7 +32,7 @@ public class Sequence extends SchemaObjectBase {
     private long maxValue;
     private boolean cycle;
     private boolean belongsToTable;
-    private Object flushSync = new Object();
+    private final Object flushSync = new Object();
     private boolean writeWithMargin;
 
     /**
@@ -208,7 +207,7 @@ public class Sequence extends SchemaObjectBase {
 
     @Override
     public String getCreateSQLForCopy(Table table, String quotedName) {
-        throw DbException.throwInternalError();
+        throw DbException.throwInternalError(toString());
     }
 
     @Override
@@ -291,36 +290,34 @@ public class Sequence extends SchemaObjectBase {
             return;
         }
         if (session == null || !database.isSysTableLockedBy(session)) {
-            // This session may not lock the sys table (except if it already has
+            // This session may not lock the sys table (except if it has already
             // locked it) because it must be committed immediately, otherwise
             // other threads can not access the sys table.
             Session sysSession = database.getSystemSession();
-            synchronized (sysSession) {
-                synchronized (flushSync) {
-                    flushInternal(sysSession);
-                }
+            synchronized (database.isMultiThreaded() ?  sysSession : database) {
+                flushInternal(sysSession);
                 sysSession.commit(false);
             }
         } else {
-            synchronized (session) {
-                synchronized (flushSync) {
-                    flushInternal(session);
-                }
+            synchronized (database.isMultiThreaded() ? session : database) {
+                flushInternal(session);
             }
         }
     }
 
     private void flushInternal(Session session) {
-        final boolean metaWasLocked = database.lockMeta(session);
-        // just for this case, use the value with the margin
-        try {
-            writeWithMargin = true;
-            database.updateMeta(session, this);
-        } finally {
-            writeWithMargin = false;
-        }
-        if (!metaWasLocked) {
-            database.unlockMeta(session);
+        synchronized (flushSync) {
+            final boolean metaWasLocked = database.lockMeta(session);
+            // just for this case, use the value with the margin
+            try {
+                writeWithMargin = true;
+                database.updateMeta(session, this);
+            } finally {
+                writeWithMargin = false;
+            }
+            if (!metaWasLocked) {
+                database.unlockMeta(session);
+            }
         }
     }
 
