@@ -32,7 +32,6 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Random;
-
 import org.h2.api.ErrorCode;
 import org.h2.bnf.Bnf;
 import org.h2.bnf.context.DbColumn;
@@ -660,10 +659,9 @@ public class WebApp {
                 treeIndex = addColumns(mainSchema, view, buff,
                         treeIndex, notManyTables, columnsBuffer);
                 if (schema.getContents().isH2()) {
-                    PreparedStatement prep = null;
-                    try {
-                        prep = conn.prepareStatement("SELECT * FROM " +
-                                "INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME=?");
+
+                    try (PreparedStatement prep = conn.prepareStatement("SELECT * FROM " +
+                                "INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME=?")) {
                         prep.setString(1, view.getName());
                         ResultSet rs = prep.executeQuery();
                         if (rs.next()) {
@@ -675,8 +673,6 @@ public class WebApp {
                             treeIndex++;
                         }
                         rs.close();
-                    } finally {
-                        JdbcUtils.closeSilently(prep);
                     }
                 }
                 buff.append("addTable('" +
@@ -717,9 +713,7 @@ public class WebApp {
                 treeIndex = addTablesAndViews(schema, false, buff, treeIndex);
             }
             if (isH2) {
-                Statement stat = null;
-                try {
-                    stat = conn.createStatement();
+                try (Statement stat = conn.createStatement()) {
                     ResultSet rs = stat.executeQuery("SELECT * FROM " +
                             "INFORMATION_SCHEMA.SEQUENCES ORDER BY SEQUENCE_NAME");
                     for (int i = 0; rs.next(); i++) {
@@ -773,8 +767,6 @@ public class WebApp {
                         }
                     }
                     rs.close();
-                } finally {
-                    JdbcUtils.closeSilently(stat);
                 }
             }
             DatabaseMetaData meta = session.getMetaData();
