@@ -13,10 +13,12 @@ import java.nio.channels.FileChannel;
 import java.nio.channels.FileLock;
 import java.nio.channels.NonWritableChannelException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
+import java.util.concurrent.atomic.AtomicReference;
 
 import org.h2.api.ErrorCode;
 import org.h2.compress.CompressLZF;
@@ -303,6 +305,19 @@ class FileMem extends FileBase {
         return this;
     }
 
+    @Override
+    public int write(ByteBuffer src, long position) throws IOException {
+        int len = src.remaining();
+        if (len == 0) {
+            return 0;
+        }
+        data.touch(readOnly);
+        data.readWrite(position, src.array(),
+                src.arrayOffset() + src.position(), len, true);
+        src.position(src.position() + len);
+        return len;
+    }
+
     @Override
     public int write(ByteBuffer src) throws IOException {
         int len = src.remaining();
@@ -316,6 +331,22 @@ class FileMem extends FileBase {
         return len;
     }
 
+    @Override
+    public int read(ByteBuffer dst, long position) throws IOException {
+        int len = dst.remaining();
+        if (len == 0) {
+            return 0;
+        }
+        long newPos = data.readWrite(position, dst.array(),
+                dst.arrayOffset() + dst.position(), len, false);
+        len = (int) (newPos - position);
+        if (len <= 0) {
+            return -1;
+        }
+        dst.position(dst.position() + len);
+        return len;
+    }
+
     @Override
     public int read(ByteBuffer dst) throws IOException {
         int len = dst.remaining();
@@ -405,7 +436,7 @@ class FileMemData {
     private final int id;
     private final boolean compress;
     private long length;
-    private byte[][] data;
+    private AtomicReference<byte[]>[] data;
     private long lastModified;
     private boolean isReadOnly;
     private boolean isLockedExclusive;
@@ -418,14 +449,50 @@ class FileMemData {
         System.arraycopy(BUFFER, 0, COMPRESSED_EMPTY_BLOCK, 0, len);
     }
 
+    @SuppressWarnings("unchecked")
     FileMemData(String name, boolean compress) {
         this.name = name;
         this.id = name.hashCode();
         this.compress = compress;
-        data = new byte[0][];
+        this.data = new AtomicReference[0];
         lastModified = System.currentTimeMillis();
     }
 
+    /**
+     * Get the page if it exists.
+     *
+     * @param page the page id
+     * @return the byte array, or null
+     */
+    byte[] getPage(int page) {
+        AtomicReference<byte[]>[] b = data;
+        if (page >= b.length) {
+            return null;
+        }
+        return b[page].get();
+    }
+
+    /**
+     * Set the page data.
+     *
+     * @param page the page id
+     * @param oldData the old data
+     * @param newData the new data
+     * @param force whether the data should be overwritten even if the old data
+     *            doesn't match
+     */
+    void setPage(int page, byte[] oldData, byte[] newData, boolean force) {
+        AtomicReference<byte[]>[] b = data;
+        if (page >= b.length) {
+            return;
+        }
+        if (force) {
+            b[page].set(newData);
+        } else {
+            b[page].compareAndSet(oldData, newData);
+        }
+    }
+
     int getId() {
         return id;
     }
@@ -480,13 +547,18 @@ class FileMemData {
             this.size = size;
         }
 
+        @Override
+        public synchronized V put(K key, V value) {
+            return super.put(key, value);
+        }
+
         @Override
         protected boolean removeEldestEntry(Map.Entry<K, V> eldest) {
             if (size() < size) {
                 return false;
             }
             CompressItem c = (CompressItem) eldest.getKey();
-            c.file.compress(c.page, c.data);
+            c.file.compress(c.page);
             return true;
         }
     }
@@ -501,11 +573,6 @@ class FileMemData {
          */
         FileMemData file;
 
-        /**
-         * The data array.
-         */
-        byte[] data;
-
         /**
          * The page to compress.
          */
@@ -531,16 +598,15 @@ class FileMemData {
         CompressItem c = new CompressItem();
         c.file = this;
         c.page = page;
-        c.data = data[page];
         synchronized (LZF) {
             COMPRESS_LATER.put(c, c);
         }
     }
 
-    private void expand(int page) {
-        byte[] d = data[page];
+    private byte[] expand(int page) {
+        byte[] d = getPage(page);
         if (d.length == BLOCK_SIZE) {
-            return;
+            return d;
         }
         byte[] out = new byte[BLOCK_SIZE];
         if (d != COMPRESSED_EMPTY_BLOCK) {
@@ -548,35 +614,28 @@ class FileMemData {
                 LZF.expand(d, 0, d.length, out, 0, BLOCK_SIZE);
             }
         }
-        data[page] = out;
+        setPage(page, d, out, false);
+        return out;
     }
 
     /**
      * Compress the data in a byte array.
      *
-     * @param data the page array
      * @param page which page to compress
      */
-    void compress(int page, byte[] old) {
-        byte[][] array = data;
-        if (page >= array.length) {
-            return;
-        }
-        byte[] d = array[page];
-        if (d != old) {
+    void compress(int page) {
+        byte[] old = getPage(page);
+        if (old == null || old.length != BLOCK_SIZE) {
+            // not yet initialized or already compressed
             return;
         }
         synchronized (LZF) {
-            int len = LZF.compress(d, BLOCK_SIZE, BUFFER, 0);
+            int len = LZF.compress(old, BLOCK_SIZE, BUFFER, 0);
             if (len <= BLOCK_SIZE) {
-                d = new byte[len];
+                byte[] d = new byte[len];
                 System.arraycopy(BUFFER, 0, d, 0, len);
                 // maybe data was changed in the meantime
-                byte[] o = array[page];
-                if (o != old) {
-                    return;
-                }
-                array[page] = d;
+                setPage(page, old, d, false);
             }
         }
     }
@@ -612,11 +671,12 @@ class FileMemData {
         long end = MathUtils.roundUpLong(newLength, BLOCK_SIZE);
         if (end != newLength) {
             int lastPage = (int) (newLength >>> BLOCK_SIZE_SHIFT);
-            expand(lastPage);
-            byte[] d = data[lastPage];
+            byte[] d = expand(lastPage);
+            byte[] d2 = Arrays.copyOf(d, d.length);
             for (int i = (int) (newLength & BLOCK_SIZE_MASK); i < BLOCK_SIZE; i++) {
-                d[i] = 0;
+                d2[i] = 0;
             }
+            setPage(lastPage, d, d2, true);
             if (compress) {
                 compressLater(lastPage);
             }
@@ -628,10 +688,9 @@ class FileMemData {
         len = MathUtils.roundUpLong(len, BLOCK_SIZE);
         int blocks = (int) (len >>> BLOCK_SIZE_SHIFT);
         if (blocks != data.length) {
-            byte[][] n = new byte[blocks][];
-            System.arraycopy(data, 0, n, 0, Math.min(data.length, n.length));
+            AtomicReference<byte[]>[] n = Arrays.copyOf(data, blocks);
             for (int i = data.length; i < blocks; i++) {
-                n[i] = COMPRESSED_EMPTY_BLOCK;
+                n[i] = new AtomicReference<byte[]>(COMPRESSED_EMPTY_BLOCK);
             }
             data = n;
         }
@@ -659,11 +718,12 @@ class FileMemData {
         while (len > 0) {
             int l = (int) Math.min(len, BLOCK_SIZE - (pos & BLOCK_SIZE_MASK));
             int page = (int) (pos >>> BLOCK_SIZE_SHIFT);
-            expand(page);
-            byte[] block = data[page];
+            byte[] block = expand(page);
             int blockOffset = (int) (pos & BLOCK_SIZE_MASK);
             if (write) {
-                System.arraycopy(b, off, block, blockOffset, l);
+                byte[] p2 = Arrays.copyOf(block, block.length);
+                System.arraycopy(b, off, p2, blockOffset, l);
+                setPage(page, block, p2, true);
             } else {
                 System.arraycopy(block, blockOffset, b, off, l);
             }
