@@ -6,6 +6,7 @@
 package org.h2.table;
 
 import java.sql.ResultSetMetaData;
+import java.util.Arrays;
 import org.h2.api.ErrorCode;
 import org.h2.command.Parser;
 import org.h2.engine.Constants;
@@ -26,6 +27,7 @@ import org.h2.util.StringUtils;
 import org.h2.value.DataType;
 import org.h2.value.Value;
 import org.h2.value.ValueDate;
+import org.h2.value.ValueEnum;
 import org.h2.value.ValueInt;
 import org.h2.value.ValueLong;
 import org.h2.value.ValueNull;
@@ -66,6 +68,7 @@ public class Column {
     private final int type;
     private long precision;
     private int scale;
+    private String[] enumerators;
     private int displaySize;
     private Table table;
     private String name;
@@ -86,13 +89,23 @@ public class Column {
     private SingleColumnResolver resolver;
     private String comment;
     private boolean primaryKey;
+    private boolean visible = true;
 
     public Column(String name, int type) {
-        this(name, type, -1, -1, -1);
+        this(name, type, -1, -1, -1, null);
+    }
+
+    public Column(String name, int type, String[] enumerators) {
+        this(name, type, -1, -1, -1, enumerators);
     }
 
     public Column(String name, int type, long precision, int scale,
             int displaySize) {
+        this(name, type, precision, scale, displaySize, null);
+    }
+
+    public Column(String name, int type, long precision, int scale,
+            int displaySize, String[] enumerators) {
         this.name = name;
         this.type = type;
         if (precision == -1 && scale == -1 && displaySize == -1 && type != Value.UNKNOWN) {
@@ -104,6 +117,7 @@ public class Column {
         this.precision = precision;
         this.scale = scale;
         this.displaySize = displaySize;
+        this.enumerators = enumerators;
     }
 
     @Override
@@ -132,8 +146,12 @@ public class Column {
         return table.getId() ^ name.hashCode();
     }
 
+    public boolean isEnumerated() {
+        return type == Value.ENUM;
+    }
+
     public Column getClone() {
-        Column newColumn = new Column(name, type, precision, scale, displaySize);
+        Column newColumn = new Column(name, type, precision, scale, displaySize, enumerators);
         newColumn.copy(this);
         return newColumn;
     }
@@ -257,6 +275,22 @@ public class Column {
         nullable = b;
     }
 
+    public String[] getEnumerators() {
+        return enumerators;
+    }
+
+    public void setEnumerators(String[] enumerators) {
+        this.enumerators = enumerators;
+    }
+
+    public boolean getVisible() {
+        return visible;
+    }
+
+    public void setVisible(boolean b) {
+        visible = b;
+    }
+
     /**
      * Validate the value, convert it if required, and update the sequence value
      * if required. If the value is null, the default value (NULL if no default
@@ -336,6 +370,18 @@ public class Column {
                         getCreateSQL(), s + " (" + value.getPrecision() + ")");
             }
         }
+        if (isEnumerated()) {
+            if (!ValueEnum.isValid(enumerators, value)) {
+                String s = value.getTraceSQL();
+                if (s.length() > 127) {
+                    s = s.substring(0, 128) + "...";
+                }
+                throw DbException.get(ErrorCode.ENUM_VALUE_NOT_PERMITTED_1,
+                        getCreateSQL(), s);
+            }
+
+            value = ValueEnum.get(enumerators, value);
+        }
         updateSequenceIfRequired(session, value);
         return value;
     }
@@ -383,7 +429,7 @@ public class Column {
         while (true) {
             ValueUuid uuid = ValueUuid.getNewRandom();
             String s = uuid.getString();
-            s = s.replace('-', '_').toUpperCase();
+            s = StringUtils.toUpperEnglish(s.replace('-', '_'));
             sequenceName = "SYSTEM_SEQUENCE_" + s;
             if (schema.findSequence(sequenceName) == null) {
                 break;
@@ -405,7 +451,8 @@ public class Column {
      */
     public void prepareExpression(Session session) {
         if (defaultExpression != null) {
-            computeTableFilter = new TableFilter(session, table, null, false, null, 0);
+            computeTableFilter = new TableFilter(session, table, null, false, null, 0,
+                    null);
             defaultExpression.mapColumns(computeTableFilter, 0);
             defaultExpression = defaultExpression.optimize(session);
         }
@@ -424,6 +471,15 @@ public class Column {
             case Value.DECIMAL:
                 buff.append('(').append(precision).append(", ").append(scale).append(')');
                 break;
+            case Value.ENUM:
+                buff.append('(');
+                for (int i = 0; i < enumerators.length; i++) {
+                    buff.append('\'').append(enumerators[i]).append('\'');
+                    if(i < enumerators.length - 1) {
+                        buff.append(',');
+                    }
+                }
+                buff.append(')');
             case Value.BYTES:
             case Value.STRING:
             case Value.STRING_IGNORECASE:
@@ -435,6 +491,11 @@ public class Column {
             default:
             }
         }
+
+        if (!visible) {
+            buff.append(" INVISIBLE ");
+        }
+
         if (defaultExpression != null) {
             String sql = defaultExpression.getSQL();
             if (sql != null) {
@@ -733,6 +794,8 @@ public class Column {
         displaySize = source.displaySize;
         name = source.name;
         precision = source.precision;
+        enumerators = source.enumerators == null ? null :
+            Arrays.copyOf(source.enumerators, source.enumerators.length);
         scale = source.scale;
         // table is not set
         // columnId is not set
@@ -747,6 +810,7 @@ public class Column {
         isComputed = source.isComputed;
         selectivity = source.selectivity;
         primaryKey = source.primaryKey;
+        visible = source.visible;
     }
 
 }
