@@ -14,14 +14,14 @@ import java.util.concurrent.Future;
 import org.h2.command.dml.Query;
 import org.h2.command.dml.Select;
 import org.h2.command.dml.SelectUnion;
+import org.h2.index.BaseIndex;
 import org.h2.index.Cursor;
-import org.h2.index.Index;
 import org.h2.index.IndexCursor;
 import org.h2.index.IndexLookupBatch;
 import org.h2.index.ViewCursor;
 import org.h2.index.ViewIndex;
 import org.h2.message.DbException;
-import org.h2.result.LocalResult;
+import org.h2.result.ResultInterface;
 import org.h2.result.Row;
 import org.h2.result.SearchRow;
 import org.h2.util.DoneFuture;
@@ -33,7 +33,7 @@ import org.h2.value.ValueLong;
 /**
  * Support for asynchronous batched index lookups on joins.
  *
- * @see Index#createLookupBatch(org.h2.table.TableFilter[], int)
+ * @see BaseIndex#createLookupBatch(org.h2.table.TableFilter[], int)
  * @see IndexLookupBatch
  * @author Sergi Vladykin
  */
@@ -167,6 +167,9 @@ public final class JoinBatch {
      * @return column value for current row
      */
     public Value getValue(int filterId, Column column) {
+        if (current == null) {
+            return null;
+        }
         Object x = current.row(filterId);
         assert x != null;
         Row row = current.isRow(filterId) ? (Row) x : ((Cursor) x).get();
@@ -738,6 +741,7 @@ public final class JoinBatch {
 
     /**
      * Simple singleton list.
+     * @param <E> Element type.
      */
     static final class SingletonList<E> extends AbstractList<E> {
         private E element;
@@ -763,6 +767,7 @@ public final class JoinBatch {
 
     /**
      * Base class for SELECT and SELECT UNION view index lookup batches.
+     * @param <R> Runner type.
      */
     private abstract static class ViewIndexLookupBatchBase<R extends QueryRunnerBase>
             implements IndexLookupBatch {
@@ -850,14 +855,15 @@ public final class JoinBatch {
     }
 
     /**
-     * Lazy query runner base.
+     * Lazy query runner base for subqueries and views.
      */
     private abstract static class QueryRunnerBase extends LazyFuture<Cursor> {
         protected final ViewIndex viewIndex;
         protected SearchRow first;
         protected SearchRow last;
+        private boolean isLazyResult;
 
-        public QueryRunnerBase(ViewIndex viewIndex) {
+        QueryRunnerBase(ViewIndex viewIndex) {
             this.viewIndex = viewIndex;
         }
 
@@ -867,6 +873,9 @@ public final class JoinBatch {
 
         @Override
         public final boolean reset() {
+            if (isLazyResult) {
+                resetViewTopFutureCursorAfterQuery();
+            }
             if (super.reset()) {
                 return true;
             }
@@ -875,11 +884,14 @@ public final class JoinBatch {
             return false;
         }
 
-        protected final ViewCursor newCursor(LocalResult localResult) {
+        protected final ViewCursor newCursor(ResultInterface localResult) {
+            isLazyResult = localResult.isLazy();
             ViewCursor cursor = new ViewCursor(viewIndex, localResult, first, last);
             clear();
             return cursor;
         }
+
+        protected abstract void resetViewTopFutureCursorAfterQuery();
     }
 
     /**
@@ -924,12 +936,12 @@ public final class JoinBatch {
     }
 
     /**
-     * Query runner.
+     * Query runner for SELECT.
      */
     private final class QueryRunner extends QueryRunnerBase {
         Future<Cursor> topFutureCursor;
 
-        public QueryRunner(ViewIndex viewIndex) {
+        QueryRunner(ViewIndex viewIndex) {
             super(viewIndex);
         }
 
@@ -948,14 +960,23 @@ public final class JoinBatch {
             }
             viewIndex.setupQueryParameters(viewIndex.getSession(), first, last, null);
             JoinBatch.this.viewTopFutureCursor = topFutureCursor;
-            LocalResult localResult;
+            ResultInterface localResult;
+            boolean lazy = false;
             try {
                 localResult = viewIndex.getQuery().query(0);
+                lazy = localResult.isLazy();
             } finally {
-                JoinBatch.this.viewTopFutureCursor = null;
+                if (!lazy) {
+                    resetViewTopFutureCursorAfterQuery();
+                }
             }
             return newCursor(localResult);
         }
+
+        @Override
+        protected void resetViewTopFutureCursorAfterQuery() {
+            JoinBatch.this.viewTopFutureCursor = null;
+        }
     }
 
     /**
@@ -1056,7 +1077,7 @@ public final class JoinBatch {
         private ViewIndexLookupBatchUnion batchUnion;
 
         @SuppressWarnings("unchecked")
-        public QueryRunnerUnion(ViewIndexLookupBatchUnion batchUnion) {
+        QueryRunnerUnion(ViewIndexLookupBatchUnion batchUnion) {
             super(batchUnion.viewIndex);
             this.batchUnion = batchUnion;
             topFutureCursors = new Future[batchUnion.filters.size()];
@@ -1078,16 +1099,29 @@ public final class JoinBatch {
                 assert topFutureCursors[i] != null;
                 joinBatches.get(i).viewTopFutureCursor = topFutureCursors[i];
             }
-            LocalResult localResult;
+            ResultInterface localResult;
+            boolean lazy = false;
             try {
                 localResult = viewIndex.getQuery().query(0);
+                lazy = localResult.isLazy();
             } finally {
-                for (int i = 0, size = joinBatches.size(); i < size; i++) {
-                    joinBatches.get(i).viewTopFutureCursor = null;
+                if (!lazy) {
+                    resetViewTopFutureCursorAfterQuery();
                 }
             }
             return newCursor(localResult);
         }
+
+        @Override
+        protected void resetViewTopFutureCursorAfterQuery() {
+            ArrayList<JoinBatch> joinBatches = batchUnion.joinBatches;
+            if (joinBatches == null) {
+                return;
+            }
+            for (int i = 0, size = joinBatches.size(); i < size; i++) {
+                joinBatches.get(i).viewTopFutureCursor = null;
+            }
+        }
     }
 }
 
