@@ -445,7 +445,8 @@ public class TableLink extends Table {
 
     @Override
     public synchronized long getRowCount(Session session) {
-        String sql = "SELECT COUNT(*) FROM " + qualifiedTableName;
+        //The foo alias is used to support the PostgreSQL syntax
+        String sql = "SELECT COUNT(*) FROM " + qualifiedTableName + " as foo";
         try {
             PreparedStatement prep = execute(sql, null, false);
             ResultSet rs = prep.getResultSet();
@@ -566,8 +567,8 @@ public class TableLink extends Table {
     }
 
     @Override
-    public String getTableType() {
-        return Table.TABLE_LINK;
+    public TableType getTableType() {
+        return TableType.TABLE_LINK;
     }
 
     @Override
