@@ -158,18 +158,25 @@ public abstract class Value {
      * The value type for string values with a fixed size.
      */
     public static final int GEOMETRY = 22;
+
     /**
      * 23 was a short-lived experiment "TIMESTAMP UTC" which has been removed.
      */
+
     /**
-     * The value type for TIMESTAMP WITH TIMEZONE values.
+     * The value type for TIMESTAMP WITH TIME ZONE values.
      */
     public static final int TIMESTAMP_TZ = 24;
 
+    /**
+     * The value type for ENUM values.
+     */
+    public static final int ENUM = 25;
+
     /**
      * The number of value types.
      */
-    public static final int TYPE_COUNT = TIMESTAMP_TZ;
+    public static final int TYPE_COUNT = ENUM;
 
     private static SoftReference<Value[]> softCache =
             new SoftReference<Value[]>(null);
@@ -272,56 +279,61 @@ public abstract class Value {
     static int getOrder(int type) {
         switch (type) {
         case UNKNOWN:
-            return 1;
+            return 1_000;
         case NULL:
-            return 2;
+            return 2_000;
         case STRING:
-            return 10;
+            return 10_000;
         case CLOB:
-            return 11;
+            return 11_000;
         case STRING_FIXED:
-            return 12;
+            return 12_000;
         case STRING_IGNORECASE:
-            return 13;
+            return 13_000;
         case BOOLEAN:
-            return 20;
+            return 20_000;
         case BYTE:
-            return 21;
+            return 21_000;
         case SHORT:
-            return 22;
+            return 22_000;
         case INT:
-            return 23;
+            return 23_000;
         case LONG:
-            return 24;
+            return 24_000;
         case DECIMAL:
-            return 25;
+            return 25_000;
         case FLOAT:
-            return 26;
+            return 26_000;
         case DOUBLE:
-            return 27;
+            return 27_000;
         case TIME:
-            return 30;
+            return 30_000;
         case DATE:
-            return 31;
+            return 31_000;
         case TIMESTAMP:
-            return 32;
+            return 32_000;
         case TIMESTAMP_TZ:
-            return 34;
+            return 34_000;
         case BYTES:
-            return 40;
+            return 40_000;
         case BLOB:
-            return 41;
+            return 41_000;
         case JAVA_OBJECT:
-            return 42;
+            return 42_000;
         case UUID:
-            return 43;
+            return 43_000;
         case GEOMETRY:
-            return 44;
+            return 44_000;
         case ARRAY:
-            return 50;
+            return 50_000;
         case RESULT_SET:
-            return 51;
+            return 51_000;
+        case ENUM:
+            return 52_000;
         default:
+            if (JdbcUtils.customDataTypesHandler != null) {
+                return JdbcUtils.customDataTypesHandler.getDataTypeOrder(type);
+            }
             throw DbException.throwInternalError("type:"+type);
         }
     }
@@ -465,7 +477,7 @@ public abstract class Value {
      * @param v the value to add
      * @return the result
      */
-    public Value add(Value v) {
+    public Value add(@SuppressWarnings("unused") Value v) {
         throw throwUnsupportedExceptionForType("+");
     }
 
@@ -488,7 +500,7 @@ public abstract class Value {
      * @param v the value to subtract
      * @return the result
      */
-    public Value subtract(Value v) {
+    public Value subtract(@SuppressWarnings("unused") Value v) {
         throw throwUnsupportedExceptionForType("-");
     }
 
@@ -498,7 +510,7 @@ public abstract class Value {
      * @param v the value to divide by
      * @return the result
      */
-    public Value divide(Value v) {
+    public Value divide(@SuppressWarnings("unused") Value v) {
         throw throwUnsupportedExceptionForType("/");
     }
 
@@ -508,7 +520,7 @@ public abstract class Value {
      * @param v the value to multiply with
      * @return the result
      */
-    public Value multiply(Value v) {
+    public Value multiply(@SuppressWarnings("unused") Value v) {
         throw throwUnsupportedExceptionForType("*");
     }
 
@@ -518,7 +530,7 @@ public abstract class Value {
      * @param v the value to take the modulus with
      * @return the result
      */
-    public Value modulus(Value v) {
+    public Value modulus(@SuppressWarnings("unused") Value v) {
         throw throwUnsupportedExceptionForType("%");
     }
 
@@ -613,6 +625,8 @@ public abstract class Value {
                     return ValueInt.get(getBoolean().booleanValue() ? 1 : 0);
                 case BYTE:
                     return ValueInt.get(getByte());
+                case ENUM:
+                    return ValueInt.get(getInt());
                 case SHORT:
                     return ValueInt.get(getShort());
                 case LONG:
@@ -844,6 +858,13 @@ public abstract class Value {
                 }
                 break;
             }
+            case ENUM: {
+                switch (getType()) {
+                    case INT:
+                    case STRING:
+                        return this;
+                }
+            }
             case BLOB: {
                 switch (getType()) {
                 case BYTES:
@@ -866,9 +887,8 @@ public abstract class Value {
                         java.util.UUID uuid = (java.util.UUID) object;
                         return ValueUuid.get(uuid.getMostSignificantBits(),
                                 uuid.getLeastSignificantBits());
-                    } else {
-                        throw DbException.get(ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                     }
+                    throw DbException.get(ErrorCode.DATA_CONVERSION_ERROR_1, getString());
                 case TIMESTAMP_TZ:
                     throw DbException.get(
                             ErrorCode.DATA_CONVERSION_ERROR_1, getString());
@@ -936,6 +956,7 @@ public abstract class Value {
             case JAVA_OBJECT:
                 return ValueJavaObject.getNoCopy(null,
                         StringUtils.convertHexToBytes(s.trim()), getDataHandler());
+            case ENUM:
             case STRING:
                 return ValueString.get(s);
             case STRING_IGNORECASE:
@@ -966,6 +987,9 @@ public abstract class Value {
             case GEOMETRY:
                 return ValueGeometry.get(s);
             default:
+                if (JdbcUtils.customDataTypesHandler != null) {
+                    return JdbcUtils.customDataTypesHandler.convert(this, targetType);
+                }
                 throw DbException.throwInternalError("type=" + targetType);
             }
         } catch (NumberFormatException e) {
@@ -1030,6 +1054,7 @@ public abstract class Value {
      * @param targetScale the requested scale
      * @return the value
      */
+    @SuppressWarnings("unused")
     public Value convertScale(boolean onlyToSmallerScale, int targetScale) {
         return this;
     }
@@ -1043,6 +1068,7 @@ public abstract class Value {
      * @param force true if losing numeric precision is allowed
      * @return the new value
      */
+    @SuppressWarnings("unused")
     public Value convertPrecision(long precision, boolean force) {
         return this;
     }
@@ -1098,6 +1124,7 @@ public abstract class Value {
      * @param tableId the table where this object is used
      * @return the new value or itself
      */
+    @SuppressWarnings("unused")
     public Value copy(DataHandler handler, int tableId) {
         return this;
     }
