@@ -9,6 +9,9 @@ import java.lang.management.ManagementFactory;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Properties;
+import java.util.TimerTask;
+import java.util.concurrent.TimeUnit;
+
 import org.h2.Driver;
 import org.h2.engine.Constants;
 import org.h2.store.fs.FilePathRec;
@@ -35,7 +38,9 @@ import org.h2.test.db.TestExclusive;
 import org.h2.test.db.TestFullText;
 import org.h2.test.db.TestFunctionOverload;
 import org.h2.test.db.TestFunctions;
+import org.h2.test.db.TestGeneralCommonTableQueries;
 import org.h2.test.db.TestIndex;
+import org.h2.test.db.TestIndexHints;
 import org.h2.test.db.TestLargeBlob;
 import org.h2.test.db.TestLinkedTable;
 import org.h2.test.db.TestListener;
@@ -88,6 +93,7 @@ import org.h2.test.jdbc.TestCallableStatement;
 import org.h2.test.jdbc.TestCancel;
 import org.h2.test.jdbc.TestConcurrentConnectionUsage;
 import org.h2.test.jdbc.TestConnection;
+import org.h2.test.jdbc.TestCustomDataTypesHandler;
 import org.h2.test.jdbc.TestDatabaseEventListener;
 import org.h2.test.jdbc.TestDriver;
 import org.h2.test.jdbc.TestJavaObject;
@@ -113,7 +119,9 @@ import org.h2.test.mvcc.TestMvcc2;
 import org.h2.test.mvcc.TestMvcc3;
 import org.h2.test.mvcc.TestMvcc4;
 import org.h2.test.mvcc.TestMvccMultiThreaded;
+import org.h2.test.mvcc.TestMvccMultiThreaded2;
 import org.h2.test.poweroff.TestReorderWrites;
+import org.h2.test.recover.RecoverLobTest;
 import org.h2.test.rowlock.TestRowLocks;
 import org.h2.test.server.TestAutoServer;
 import org.h2.test.server.TestInit;
@@ -223,6 +231,7 @@ import org.h2.util.New;
 import org.h2.util.Profiler;
 import org.h2.util.StringUtils;
 import org.h2.util.Task;
+import org.h2.util.ThreadDeadlockDetector;
 import org.h2.util.Utils;
 
 /**
@@ -263,6 +272,12 @@ java org.h2.test.TestAll timer
 
 */
 
+    /**
+     * Set to true if any of the tests fail. Used to return an error code from
+     * the whole program.
+     */
+    static boolean atLeastOneTestFailed;
+
     /**
      * Whether the MVStore storage is used.
      */
@@ -303,6 +318,11 @@ java org.h2.test.TestAll timer
      */
     public boolean multiThreaded;
 
+    /**
+     * If lazy queries should be used.
+     */
+    public boolean lazy;
+
     /**
      * The cipher to use (null for unencrypted).
      */
@@ -339,7 +359,7 @@ java org.h2.test.TestAll timer
     public boolean splitFileSystem;
 
     /**
-     * If only fast tests should be run. If enabled, SSL is not tested.
+     * If only fast/CI/Jenkins/Travis tests should be run.
      */
     public boolean fast;
 
@@ -421,11 +441,14 @@ java org.h2.test.TestAll timer
         run(args);
         catcher.stop();
         catcher.writeTo("Test Output", "docs/html/testOutput.html");
+        if (atLeastOneTestFailed) {
+            System.exit(1);
+        }
     }
 
     private static void run(String... args) throws Exception {
         SelfDestructor.startCountdown(4 * 60);
-        long time = System.currentTimeMillis();
+        long time = System.nanoTime();
         printSystemInfo();
 
         // use lower values, to better test those cases,
@@ -521,19 +544,21 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
             test.testAll();
         }
         System.out.println(TestBase.formatTime(
-                System.currentTimeMillis() - time) + " total");
+                TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - time)) + " total");
     }
 
     private void testAll() throws Exception {
         runTests();
-        Profiler prof = new Profiler();
-        prof.depth = 16;
-        prof.interval = 1;
-        prof.startCollecting();
-        TestPerformance.main("-init", "-db", "1", "-size", "1000");
-        prof.stopCollecting();
-        System.out.println(prof.getTop(5));
-        TestPerformance.main("-init", "-db", "1", "-size", "1000");
+        if (!fast) {
+            Profiler prof = new Profiler();
+            prof.depth = 16;
+            prof.interval = 1;
+            prof.startCollecting();
+            TestPerformance.main("-init", "-db", "1", "-size", "1000");
+            prof.stopCollecting();
+            System.out.println(prof.getTop(5));
+            TestPerformance.main("-init", "-db", "1", "-size", "1000");
+        }
     }
 
     /**
@@ -574,18 +599,40 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
 
         smallLog = big = networked = memory = ssl = false;
         diskResult = traceSystemOut = diskUndo = false;
-        mvcc = mvStore;
         traceTest = stopOnError = false;
         defrag = false;
         traceLevelFile = throttle = 0;
         cipher = null;
-        // splitFileSystem = true;
+
+        // memory is a good match for multi-threaded, makes things happen
+        // faster, more chance of exposing race conditions
+        memory = true;
+        multiThreaded = true;
         test();
         testUnit();
 
-        networked = true;
+        // lazy
+        lazy = true;
         memory = true;
-        splitFileSystem = false;
+        multiThreaded = true;
+        test();
+        lazy = false;
+
+        // but sometimes race conditions need bigger windows
+        memory = false;
+        multiThreaded = true;
+        test();
+        testUnit();
+
+        // a more normal setup
+        memory = false;
+        multiThreaded = false;
+        test();
+        testUnit();
+
+        memory = true;
+        multiThreaded = false;
+        networked = true;
         test();
 
         memory = false;
@@ -607,16 +654,14 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         defrag = true;
         test();
 
-        traceLevelFile = 0;
-        smallLog = true;
-        networked = true;
         if (!fast) {
+            traceLevelFile = 0;
+            smallLog = true;
+            networked = true;
+            defrag = false;
             ssl = true;
-        }
-        defrag = false;
-        test();
+            test();
 
-        if (!fast) {
             big = true;
             smallLog = false;
             networked = false;
@@ -624,18 +669,14 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
             traceLevelFile = 0;
             test();
             testUnit();
+
+            big = false;
+            cipher = "AES";
+            test();
+            cipher = null;
+            test();
         }
 
-        big = false;
-        cipher = "AES";
-        test();
-
-        mvcc = true;
-        cipher = null;
-        test();
-
-        memory = true;
-        test();
     }
 
     /**
@@ -685,6 +726,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestFunctions());
         addTest(new TestInit());
         addTest(new TestIndex());
+        addTest(new TestIndexHints());
         addTest(new TestLargeBlob());
         addTest(new TestLinkedTable());
         addTest(new TestListener());
@@ -698,6 +740,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestOutOfMemory());
         addTest(new TestReadOnly());
         addTest(new TestRecursiveQueries());
+        addTest(new TestGeneralCommonTableQueries());
         addTest(new TestRights());
         addTest(new TestRunscript());
         addTest(new TestSQLInjection());
@@ -746,6 +789,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestTransactionIsolation());
         addTest(new TestUpdatableResultSet());
         addTest(new TestZloty());
+        addTest(new TestCustomDataTypesHandler());
 
         // jdbcx
         addTest(new TestConnectionPool());
@@ -763,6 +807,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestMvcc3());
         addTest(new TestMvcc4());
         addTest(new TestMvccMultiThreaded());
+        addTest(new TestMvccMultiThreaded2());
         addTest(new TestRowLocks());
 
         // synth
@@ -858,6 +903,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestReader());
         addTest(new TestRecovery());
         addTest(new TestScriptReader());
+        addTest(new RecoverLobTest());
         addTest(createTest("org.h2.test.unit.TestServlet"));
         addTest(new TestSecurity());
         addTest(new TestShell());
@@ -873,6 +919,7 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         addTest(new TestValueHashMap());
         addTest(new TestWeb());
 
+
         runAddedTests();
 
         // serial
@@ -898,7 +945,21 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
         // tests.add(test);
         // run directly for now, because concurrently running tests
         // fails on Raspberry Pi quite often (seems to be a JVM problem)
-        test.runTest(this);
+
+        // event queue watchdog for tests that get stuck when running in Jenkins
+        final java.util.Timer watchdog = new java.util.Timer();
+        // 5 minutes
+        watchdog.schedule(new TimerTask() {
+            @Override
+            public void run() {
+                ThreadDeadlockDetector.dumpAllThreadsAndLocks("test watchdog timed out");
+            }
+        }, 5 * 60 * 1000);
+        try {
+            test.runTest(this);
+        } finally {
+            watchdog.cancel();
+        }
     }
 
     private void runAddedTests() {
@@ -1019,12 +1080,14 @@ kill -9 `jps -l | grep "org.h2.test." | cut -d " " -f 1`
     public String toString() {
         StringBuilder buff = new StringBuilder();
         appendIf(buff, fast, "fast");
+        appendIf(buff, lazy, "lazy");
         appendIf(buff, mvStore, "mvStore");
         appendIf(buff, big, "big");
         appendIf(buff, networked, "net");
         appendIf(buff, memory, "memory");
         appendIf(buff, codeCoverage, "codeCoverage");
         appendIf(buff, mvcc, "mvcc");
+        appendIf(buff, multiThreaded, "multiThreaded");
         appendIf(buff, cipher != null, cipher);
         appendIf(buff, cacheType != null, cacheType);
         appendIf(buff, smallLog, "smallLog");
