@@ -18,6 +18,7 @@ import java.util.ArrayList;
 import java.util.Properties;
 import java.util.Random;
 import java.util.StringTokenizer;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.h2.test.TestBase;
 import org.h2.tools.Server;
@@ -35,7 +36,7 @@ class Database {
     private String name, url, user, password;
     private final ArrayList<String[]> replace = new ArrayList<String[]>();
     private String currentAction;
-    private long startTime;
+    private long startTimeNs;
     private Connection conn;
     private Statement stat;
     private long lastTrace;
@@ -195,7 +196,7 @@ class Database {
             }
         } else if (url.startsWith("jdbc:hsqldb:")) {
             // HSQLDB: use a WRITE_DELAY of 1 second
-            try (Statement s = newConn.createStatement()){
+            try (Statement s = newConn.createStatement()) {
                 s.execute("SET WRITE_DELAY 1");
             }
         }
@@ -270,7 +271,7 @@ class Database {
      */
     void start(Bench bench, String action) {
         this.currentAction = bench.getName() + ": " + action;
-        this.startTime = System.currentTimeMillis();
+        this.startTimeNs = System.nanoTime();
     }
 
     /**
@@ -278,7 +279,7 @@ class Database {
      * data.
      */
     void end() {
-        long time = System.currentTimeMillis() - startTime;
+        long time = TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - startTimeNs);
         log(currentAction, "ms", (int) time);
         if (test.isCollect()) {
             totalTime += time;
@@ -361,10 +362,10 @@ class Database {
      */
     void trace(String action, int i, int max) {
         if (TRACE) {
-            long time = System.currentTimeMillis();
+            long time = System.nanoTime();
             if (i == 0 || lastTrace == 0) {
                 lastTrace = time;
-            } else if (time > lastTrace + 1000) {
+            } else if (time > lastTrace + TimeUnit.SECONDS.toNanos(1)) {
                 System.out.println(action + ": " + ((100 * i / max) + "%"));
                 lastTrace = time;
             }
@@ -402,9 +403,9 @@ class Database {
      * @return the result set
      */
     ResultSet query(PreparedStatement prep) throws SQLException {
-        // long time = System.currentTimeMillis();
+        // long time = System.nanoTime();
         ResultSet rs = prep.executeQuery();
-        // time = System.currentTimeMillis() - time;
+        // time = System.nanoTime() - time;
         // if(time > 100) {
         //     System.out.println("time="+time);
         // }
