@@ -50,6 +50,7 @@ public class TestCompatibility extends TestBase {
         testDB2();
         testDerby();
         testSybaseAndMSSQLServer();
+        testIgnite();
 
         conn.close();
         deleteDb("compatibility");
@@ -411,6 +412,9 @@ public class TestCompatibility extends TestBase {
         // make sure we're ignoring the index part of the statement
         rs = stat.executeQuery("select * from test (index table1_index)");
         rs.close();
+
+        // UNIQUEIDENTIFIER is MSSQL's equivalent of UUID
+        stat.execute("create table test3 (id UNIQUEIDENTIFIER)");
     }
 
     private void testDB2() throws SQLException {
@@ -484,4 +488,26 @@ public class TestCompatibility extends TestBase {
         conn.close();
         conn = getConnection("compatibility");
     }
+
+    private void testIgnite() throws SQLException {
+        Statement stat = conn.createStatement();
+        stat.execute("SET MODE Ignite");
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int affinity key)");
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int affinity primary key)");
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int, v1 varchar, v2 long affinity key, primary key(v1, id))");
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int, v1 varchar, v2 long, primary key(v1, id), affinity key (id))");
+
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int shard key)");
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int shard primary key)");
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int, v1 varchar, v2 long shard key, primary key(v1, id))");
+        stat.execute("DROP TABLE IF EXISTS TEST");
+        stat.execute("create table test(id int, v1 varchar, v2 long, primary key(v1, id), shard key (id))");
+    }
 }
