@@ -38,32 +38,58 @@ public class TestQueryCache extends TestBase {
     private void test1() throws Exception {
         Connection conn = getConnection("queryCache;QUERY_CACHE_SIZE=10");
         Statement stat = conn.createStatement();
-        stat.execute("create table test(id int, name varchar) " +
-                "as select x, space(100) from system_range(1, 1000)");
+        stat.execute("create table test(id int, name varchar)");
         PreparedStatement prep;
-        conn.prepareStatement("select count(*) from test t1, test t2");
+        // query execution may be fast here but the parsing must be slow
+        StringBuilder queryBuilder = new StringBuilder("select count(*) from test t1 where \n");
+        for (int i = 0; i < 1000; i++) {
+            if (i != 0) {
+                queryBuilder.append(" and ");
+            }
+            queryBuilder.append(" TIMESTAMP '2005-12-31 23:59:59' = TIMESTAMP '2005-12-31 23:59:59' ");
+        }
+        String query = queryBuilder.toString();
+        conn.prepareStatement(query);
+        int firstGreater = 0;
+        int firstSmaller = 0;
         long time;
         ResultSet rs;
         long first = 0;
-        for (int i = 0; i < 4; i++) {
+        // 1000 iterations to warm up and avoid JIT effects
+        for (int i = 0; i < 1005; i++) {
             // this should both ensure results are not re-used
             // stat.execute("set mode regular");
             // stat.execute("create table x()");
             // stat.execute("drop table x");
             time = System.nanoTime();
-            prep = conn.prepareStatement("select count(*) from test t1, test t2");
-            prep.executeQuery();
-            rs = stat.executeQuery("select count(*) from test t1, test t2");
+            prep = conn.prepareStatement(query);
+            execute(prep);
+            prep.close();
+            rs = stat.executeQuery(query);
             rs.next();
             int c = rs.getInt(1);
-            assertEquals(1000000, c);
+            rs.close();
+            assertEquals(0, c);
             time = System.nanoTime() - time;
-            if (first == 0) {
+            if (i == 1000) {
+                // take from cache and do not close, so that next iteration will have a cache miss
+                prep = conn.prepareStatement(query);
+            } else if (i == 1001) {
                 first = time;
-            } else {
-                assertSmaller(time, first);
+                // try to avoid pauses in subsequent iterations
+                System.gc();
+            } else if (i > 1001) {
+                if (first > time) {
+                    firstGreater++;
+                } else {
+                    firstSmaller++;
+                }
             }
         }
+        // first prepare time must be always greater because of query cache,
+        // but JVM is too unpredictable to assert that, so just check that
+        // usually this is true
+        assertSmaller(firstSmaller, firstGreater);
         stat.execute("drop table test");
         conn.close();
     }
