@@ -9,7 +9,6 @@ import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
-
 import org.h2.test.TestBase;
 
 /**
@@ -54,7 +53,7 @@ public class TestSessionsLocks extends TestBase {
         assertEquals("PUBLIC", rs.getString("TABLE_SCHEMA"));
         assertEquals("TEST", rs.getString("TABLE_NAME"));
         rs.getString("SESSION_ID");
-        if (config.mvcc) {
+        if (config.mvcc || config.mvStore) {
             assertEquals("READ", rs.getString("LOCK_TYPE"));
         } else {
             assertEquals("WRITE", rs.getString("LOCK_TYPE"));
@@ -65,7 +64,7 @@ public class TestSessionsLocks extends TestBase {
         stat2.execute("SELECT * FROM TEST");
         rs = stat.executeQuery("select * from information_schema.locks " +
                 "order by session_id");
-        if (!config.mvcc) {
+        if (!config.mvcc && !config.mvStore) {
             rs.next();
             assertEquals("PUBLIC", rs.getString("TABLE_SCHEMA"));
             assertEquals("TEST", rs.getString("TABLE_NAME"));
