@@ -94,6 +94,7 @@ public class TestSpatial extends TestBase {
         testNullableGeometryUpdate();
         testIndexUpdateNullGeometry();
         testInsertNull();
+        testSpatialIndexWithOrder();
     }
 
     private void testBug1() throws SQLException {
@@ -1009,9 +1010,9 @@ public class TestSpatial extends TestBase {
         stat.execute("create memory table test"
                 + "(id int primary key, the_geom geometry, description varchar2(32))");
         stat.execute("create spatial index on test(the_geom)");
-        stat.execute("insert into test values(1, null, null)");
-        stat.execute("insert into test values(2, null, null)");
-        stat.execute("insert into test values(3, null, null)");
+        for (int i = 0; i < 1000; i++) {
+            stat.execute("insert into test values("+ (i + 1) +", null, null)");
+        }
         ResultSet rs = stat.executeQuery("select * from test");
         assertTrue(rs.next());
         assertEquals(1, rs.getInt(1));
@@ -1138,4 +1139,21 @@ public class TestSpatial extends TestBase {
         }
 
     }
+
+    private void testSpatialIndexWithOrder() throws SQLException {
+        deleteDb("spatial");
+        Connection conn = getConnection(URL);
+        Statement stat = conn.createStatement();
+        stat.execute("DROP TABLE IF EXISTS BUILDINGS;" +
+                "CREATE TABLE BUILDINGS (PK serial, THE_GEOM geometry);" +
+                "insert into buildings(the_geom) SELECT 'POINT(1 1)" +
+                "'::geometry from SYSTEM_RANGE(1,10000);\n" +
+                "CREATE SPATIAL INDEX ON PUBLIC.BUILDINGS(THE_GEOM);\n");
+
+        try (ResultSet rs = stat.executeQuery("EXPLAIN SELECT * FROM " +
+                "BUILDINGS ORDER BY PK LIMIT 51;")) {
+            assertTrue(rs.next());
+            assertTrue(rs.getString(1).contains("PRIMARY_KEY"));
+        }
+    }
 }
