@@ -266,8 +266,13 @@ public class TestUpdatableResultSet extends TestBase {
         assertTrue(rs.absolute(3));
         assertEquals(3, rs.getRow());
 
-        assertTrue(rs.absolute(-1));
-        assertEquals(3, rs.getRow());
+        if (!config.lazy) {
+            assertTrue(rs.absolute(-1));
+            assertEquals(3, rs.getRow());
+
+            assertTrue(rs.absolute(-2));
+            assertEquals(2, rs.getRow());
+        }
 
         assertFalse(rs.absolute(4));
         assertTrue(rs.isAfterLast());
