@@ -142,7 +142,7 @@ public class TestWeb extends TestBase {
         Server server = Server.createWebServer(
                 "-webPort", "8182", "-properties", "null");
         server.start();
-        assertTrue(server.getStatus().contains("server running"));
+        assertContains(server.getStatus(), "server running");
         Server server2 = Server.createWebServer(
                 "-webPort", "8182", "-properties", "null");
         assertEquals("Not started", server2.getStatus());
@@ -150,9 +150,9 @@ public class TestWeb extends TestBase {
             server2.start();
             fail();
         } catch (Exception e) {
-            assertTrue(e.toString().contains("port may be in use"));
-            assertTrue(server2.getStatus().contains(
-                    "could not be started"));
+            assertContains(e.toString(), "port may be in use");
+            assertContains(server2.getStatus(),
+                    "could not be started");
         }
         server.stop();
     }
