@@ -10,15 +10,14 @@ import java.sql.PreparedStatement;
 import java.sql.Statement;
 import java.util.Random;
 import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
-
 import org.h2.mvstore.MVMap;
 import org.h2.mvstore.MVStore;
 import org.h2.store.FileLister;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
-import org.h2.util.Profiler;
 import org.h2.util.Task;
 
 /**
@@ -132,7 +131,8 @@ public class TestBenchmark extends TestBase {
         Statement stat;
         String url = "mvstore";
         if (mvStore) {
-            url += ";MV_STORE=TRUE"; // ;COMPRESS=TRUE";
+            // ;COMPRESS=TRUE";
+            url += ";MV_STORE=TRUE";
         }
 
         url = getURL(url, true);
@@ -158,12 +158,12 @@ public class TestBenchmark extends TestBase {
             }
         }
 
-        long start = System.currentTimeMillis();
+        long start = System.nanoTime();
         // Profiler prof = new Profiler().startCollecting();
         stat.execute("create index on test(data)");
         // System.out.println(prof.getTop(5));
 
-        System.out.println((System.currentTimeMillis() - start) + " "
+        System.out.println(TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - start) + " "
                 + (mvStore ? "mvstore" : "default"));
         conn.createStatement().execute("shutdown compact");
         conn.close();
@@ -193,7 +193,7 @@ public class TestBenchmark extends TestBase {
         int rowCount = 100;
         int readCount = 20 * rowCount;
 
-        long start = System.currentTimeMillis();
+        long start = System.nanoTime();
 
         for (int i = 0; i < rowCount; i++) {
             prep.setInt(1, i);
@@ -211,12 +211,12 @@ public class TestBenchmark extends TestBase {
             prep.executeQuery();
         }
 
-        System.out.println((System.currentTimeMillis() - start) + " "
+        System.out.println(TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - start) + " "
                 + (mvStore ? "mvstore" : "default"));
         conn.close();
     }
 
-    private void randomize(byte[] data, int i) {
+    private static void randomize(byte[] data, int i) {
         Random r = new Random(i);
         r.nextBytes(data);
     }
@@ -251,7 +251,7 @@ public class TestBenchmark extends TestBase {
                 conn.commit();
             }
         }
-        long start = System.currentTimeMillis();
+        long start = System.nanoTime();
 
         prep = conn.prepareStatement("select * from test where id = ?");
         for (int i = 0; i < readCount; i++) {
@@ -259,7 +259,7 @@ public class TestBenchmark extends TestBase {
             prep.executeQuery();
         }
 
-        System.out.println((System.currentTimeMillis() - start) + " "
+        System.out.println(TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - start) + " "
                 + (mvStore ? "mvstore" : "default"));
         conn.createStatement().execute("shutdown compact");
         conn.close();
