@@ -8,8 +8,8 @@ package org.h2.test.store;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.Random;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
-
 import org.h2.mvstore.ConcurrentArrayList;
 import org.h2.test.TestBase;
 import org.h2.util.Task;
@@ -45,9 +45,9 @@ public class TestConcurrentLinkedList extends TestBase {
         testPerformance(false);
     }
 
-    private void testPerformance(final boolean stock) {
+    private static void testPerformance(final boolean stock) {
         System.out.print(stock ? "stock " : "custom ");
-        long start = System.currentTimeMillis();
+        long start = System.nanoTime();
         // final ConcurrentLinkedList<Integer> test =
         //     new ConcurrentLinkedList<Integer>();
         final ConcurrentArrayList<Integer> test =
@@ -105,7 +105,7 @@ public class TestConcurrentLinkedList extends TestBase {
             }
         }
         task.get();
-        System.out.println(System.currentTimeMillis() - start);
+        System.out.println(TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - start));
     }
 
     private void testConcurrent() {
