@@ -19,7 +19,6 @@ import java.sql.SQLException;
 import java.sql.Savepoint;
 import java.sql.Statement;
 import java.util.concurrent.atomic.AtomicBoolean;
-
 import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
 import org.h2.engine.Database;
@@ -111,17 +110,16 @@ public class TestMVTableEngine extends TestBase {
 
     private void testLobReuse() throws Exception {
         deleteDb(getTestName());
-        Connection conn = getConnection(getTestName());
-        Statement stat = conn.createStatement();
+        Connection conn1 = getConnection(getTestName());
+        Statement stat = conn1.createStatement();
         stat.execute("create table test(id identity primary key, lob clob)");
-        conn.close();
         byte[] buffer = new byte[8192];
         for (int i = 0; i < 20; i++) {
-            conn = getConnection(getTestName());
-            stat = conn.createStatement();
+            Connection conn2 = getConnection(getTestName());
+            stat = conn2.createStatement();
             stat.execute("insert into test(lob) select space(1025) from system_range(1, 10)");
             stat.execute("delete from test where random() > 0.5");
-            ResultSet rs = conn.createStatement().executeQuery(
+            ResultSet rs = conn2.createStatement().executeQuery(
                     "select lob from test");
             while (rs.next()) {
                 InputStream is = rs.getBinaryStream(1);
@@ -129,11 +127,15 @@ public class TestMVTableEngine extends TestBase {
                     // ignore
                 }
             }
-            conn.close();
+            conn2.close();
         }
+        conn1.close();
     }
 
     private void testShutdownDuringLobCreation() throws Exception {
+        if (config.memory) {
+            return;
+        }
         deleteDb(getTestName());
         Connection conn = getConnection(getTestName());
         Statement stat = conn.createStatement();
@@ -201,6 +203,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testLobCreationThenShutdown() throws Exception {
+        if (config.memory) {
+            return;
+        }
         deleteDb(getTestName());
         Connection conn = getConnection(getTestName());
         Statement stat = conn.createStatement();
@@ -259,6 +264,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testAppendOnly() throws Exception {
+        if (config.memory) {
+            return;
+        }
         deleteDb(getTestName());
         Connection conn = getConnection(getTestName());
         Statement stat = conn.createStatement();
@@ -329,6 +337,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testOldAndNew() throws SQLException {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         deleteDb(getTestName());
         String urlOld = getURL(getTestName() + ";MV_STORE=FALSE", true);
@@ -422,6 +433,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testGarbageCollectionForLOB() throws SQLException {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         Statement stat;
         deleteDb(getTestName());
@@ -528,7 +542,7 @@ public class TestMVTableEngine extends TestBase {
         rs.next();
         plan = rs.getString(1);
         // transaction log is larger than the table, so read the table
-        assertTrue(plan, plan.contains("reads:"));
+        assertContains(plan, "reads:");
         rs = stat2.executeQuery("select count(*) from test");
         rs.next();
         assertEquals(10000, rs.getInt(1));
@@ -656,6 +670,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testShrinkDatabaseFile() throws Exception {
+        if (config.memory) {
+            return;
+        }
         deleteDb(getTestName());
         String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
@@ -719,6 +736,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testTwoPhaseCommit() throws Exception {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         Statement stat;
         deleteDb(getTestName());
@@ -746,6 +766,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testRecover() throws Exception {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         Statement stat;
         deleteDb(getTestName());
@@ -791,6 +814,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testSeparateKey() throws Exception {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         Statement stat;
         deleteDb(getTestName());
@@ -816,6 +842,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testRollbackAfterCrash() throws Exception {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         Statement stat;
         deleteDb(getTestName());
@@ -947,6 +976,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testWriteDelay() throws Exception {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         Statement stat;
         ResultSet rs;
@@ -1004,6 +1036,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testReopen() throws SQLException {
+        if (config.memory) {
+            return;
+        }
         Connection conn;
         Statement stat;
         deleteDb(getTestName());
@@ -1018,6 +1053,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testBlob() throws SQLException, IOException {
+        if (config.memory) {
+            return;
+        }
         deleteDb(getTestName());
         String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
@@ -1045,6 +1083,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testEncryption() throws Exception {
+        if (config.memory) {
+            return;
+        }
         deleteDb(getTestName());
         String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
@@ -1087,6 +1128,9 @@ public class TestMVTableEngine extends TestBase {
     }
 
     private void testReadOnly() throws Exception {
+        if (config.memory) {
+            return;
+        }
         deleteDb(getTestName());
         String dbName = getTestName() + ";MV_STORE=TRUE";
         Connection conn;
