@@ -1,12 +1,13 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
- * Initial Developer: H2 Group
+ * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0, and the
+ * EPL 1.0 (http://h2database.com/html/license.html). Initial Developer: H2
+ * Group
  */
 package org.h2.test.unit;
 
 import java.util.Locale;
 import java.util.Random;
+import java.util.concurrent.TimeUnit;
 
 import org.h2.test.TestBase;
 import org.h2.util.StringUtils;
@@ -23,7 +24,6 @@ public class TestStringCache extends TestBase {
     private final Random random = new Random(1);
     private final String[] some = { null, "", "ABC",
             "this is a medium sized string", "1", "2" };
-    private boolean returnNew;
     private boolean useIntern;
 
     /**
@@ -40,11 +40,6 @@ public class TestStringCache extends TestBase {
     @Override
     public void test() throws InterruptedException {
         testToUpperToLower();
-        returnNew = true;
-        StringUtils.clearCache();
-        testSingleThread(getSize(5000, 20000));
-        testMultiThreads();
-        returnNew = false;
         StringUtils.clearCache();
         testSingleThread(getSize(5000, 20000));
         testMultiThreads();
@@ -63,24 +58,24 @@ public class TestStringCache extends TestBase {
         }
         int repeat = 100000;
         int testLen = 0;
-        long time = System.currentTimeMillis();
+        long time = System.nanoTime();
         for (int a = 0; a < repeat; a++) {
             for (String x : test) {
                 String y = StringUtils.toUpperEnglish(x);
                 testLen += y.length();
             }
         }
-        time = System.currentTimeMillis() - time;
-        System.out.println("cache " + time);
-        time = System.currentTimeMillis();
+        time = System.nanoTime() - time;
+        System.out.println("cache " + TimeUnit.NANOSECONDS.toMillis(time));
+        time = System.nanoTime();
         for (int a = 0; a < repeat; a++) {
             for (String x : test) {
                 String y = x.toUpperCase(Locale.ENGLISH);
                 testLen -= y.length();
             }
         }
-        time = System.currentTimeMillis() - time;
-        System.out.println("toUpperCase " + time);
+        time = System.nanoTime() - time;
+        System.out.println("toUpperCase " + TimeUnit.NANOSECONDS.toMillis(time));
         assertEquals(0, testLen);
     }
 
@@ -105,13 +100,13 @@ public class TestStringCache extends TestBase {
         testToUpperCache();
         testToUpperCache();
         testToUpperCache();
-        returnNew = false;
         for (int i = 0; i < 6; i++) {
             useIntern = (i % 2) == 0;
-            long time = System.currentTimeMillis();
+            long time = System.nanoTime();
             testSingleThread(100000);
-            time = System.currentTimeMillis() - time;
-            System.out.println(time + " ms (useIntern=" + useIntern + ")");
+            time = System.nanoTime() - time;
+            System.out.println(TimeUnit.NANOSECONDS.toMillis(time) +
+                    " ms (useIntern=" + useIntern + ")");
         }
 
     }
@@ -124,7 +119,8 @@ public class TestStringCache extends TestBase {
             }
             return s;
         }
-        int len = random.nextBoolean() ? random.nextInt(1000) : random.nextInt(10);
+        int len = random.nextBoolean() ? random.nextInt(1000)
+                : random.nextInt(10);
         StringBuilder buff = new StringBuilder(len);
         for (int i = 0; i < len; i++) {
             buff.append(random.nextInt(0xfff));
@@ -137,29 +133,16 @@ public class TestStringCache extends TestBase {
      */
     void testString() {
         String a = randomString();
-        if (returnNew) {
-            String b = StringUtils.fromCacheOrNew(a);
-            try {
-                assertEquals(a, b);
-            } catch (Exception e) {
-                TestBase.logError("error", e);
-            }
-            if (a != null && a == b && a.length() > 0) {
-                throw new AssertionError("a=" + System.identityHashCode(a) +
-                        " b=" + System.identityHashCode(b));
-            }
+        String b;
+        if (useIntern) {
+            b = a == null ? null : a.intern();
         } else {
-            String b;
-            if (useIntern) {
-                b = a == null ? null : a.intern();
-            } else {
-                b = StringUtils.cache(a);
-            }
-            try {
-                assertEquals(a, b);
-            } catch (Exception e) {
-                TestBase.logError("error", e);
-            }
+            b = StringUtils.cache(a);
+        }
+        try {
+            assertEquals(a, b);
+        } catch (Exception e) {
+            TestBase.logError("error", e);
         }
     }
 
