@@ -138,6 +138,12 @@ public class FilePathReorderWrites extends FilePathWrapper {
         return 45000;
     }
 
+    @Override
+    public void delete() {
+        super.delete();
+        FilePath.get(getBase().toString() + ".copy").delete();
+    }
+
 }
 
 /**
@@ -277,9 +283,8 @@ class FileReorderWrites extends FileBase {
             x += addOperation(
                     new FileWriteOperation(id++, position + len1, buf2));
             return x;
-        } else {
-            return addOperation(new FileWriteOperation(id++, position, src));
         }
+        return addOperation(new FileWriteOperation(id++, position, src));
     }
 
     private void checkError() throws IOException {
@@ -378,7 +383,7 @@ class FileReorderWrites extends FileBase {
                 channel.truncate(position);
                 return -1;
             }
-            // TODO support the case were part is not written
+            // TODO support the case where part is not written
             int len = channel.write(buffer, position);
             buffer.flip();
             return len;
