@@ -18,7 +18,7 @@ public class H2Utils {
      * @param factory the cursor factory to use
      * @return the database connection
      */
-    public static H2Database openOrCreateDatabase(String name, int mode,
+    public static H2Database openOrCreateDatabase(String name, @SuppressWarnings("unused") int mode,
             H2Database.CursorFactory factory) {
         return H2Database.openOrCreateDatabase(name, factory);
     }
